/*
 * TkResolver++ - Psychokinesis testing software
 *
 * Copyright (C) 2007 Robert Ham <rah@bash.sh>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef __TK_VALUE_H__
#define __TK_VALUE_H__

#include <stdexcept>

#include "tkstring.h"

namespace Tk {
  
  template <typename T>
    class Value
    {
    private:
      T    *_value;
      
    public:
      bool is_null() const
	{ return _value == 0; }
      bool is_valid() const
	{ return !is_null(); }
      
    private:
      void check_null() const
	{ if (is_null()) throw std::runtime_error("value is null"); }
      
    public:
      Value()
	: _value(0) {}
      Value(const T &t)
	: _value(new T(t)) {}
      Value(const T *t)
	: _value(t ? new T(*t) : 0) {}
      Value(const Value &v)
	: _value( v.is_null() ? 0 : new T(v.get_value()) ) {}
      
      const T *get_value_ptr() const
	{ return _value; }
      T *get_value_ptr()
	{ return _value; }
      
      T &get_value()
	{ check_null(); return *_value;}
      const T &get_value() const
	{ check_null(); return *_value;}
      
      void clear()
	{ if (is_valid()) { delete _value; _value = 0; } }
      
      Value &operator= (const Value &v)
	{ clear(); _value = v.is_null() ? 0 : new T(v.get_value()); return *this; }
      T &operator= (const T &t)
	{ clear(); _value = new T(t); return *_value; }
      T *operator= (const T *t)
	{ clear(); _value = t ? new T(*t) : 0; return _value; }
      template <typename X>
	Value &operator= (const Value<X> &v)
	{ if (v.is_null()) clear(); else *this = static_cast<T>(v.get_value()); return *this; }
      
      operator T & ()
	{ return get_value(); }
      operator const T & () const
	{ return get_value(); }
      operator T * ()
	{ return get_value_ptr(); }
      operator const T * () const
	{ return get_value_ptr(); }
      operator bool () const
	{ return is_valid(); }
    };
  
  typedef Value<int> Int;
  typedef Value<unsigned> Unsigned;
  typedef Value<long> Long;
  typedef Value<unsigned long> UnsignedLong;
  typedef Value<long long> LongLong;
  typedef Value<unsigned long> UnsignedLongLong;
  typedef Value<float> Float;
  typedef Value<double> Double;
  typedef Value<long double> LongDouble;
  typedef Value<bool> Bool;
  typedef Value<String> StringValue;
  
}; /* namespace TkValue */


#endif /* __TK_NULL_H__ */
