/* -*- c++ -*-
 * TkResolver++ - Psychokinesis testing software
 *
 * Copyright (C) 2008 Robert Ham <rah@bash.sh>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include "libtkdbui/temporal_value.h"
#include "libtkcommon/tkintl.h"

using namespace Tk::Db::Ui;

TemporalValue::DateDialog:: DateDialog(unsigned day, unsigned month, unsigned year)
  : Gtk::Dialog("Date selector", true)
{
  add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
  add_button(Gtk::Stock::OK, Gtk::RESPONSE_ACCEPT);
  
  get_vbox()->pack_start(_calendar, true, true);
  _calendar.select_month(month - 1, year);
  _calendar.select_day(day);
  _calendar.signal_day_selected_double_click().connect( sigc::bind<int>(sigc::mem_fun(*this, &Gtk::Dialog::response), Gtk::RESPONSE_ACCEPT) );
  
  show_all();
}

void
TemporalValue::DateDialog:: get_date(unsigned &day, unsigned &month, unsigned &year)
{
  unsigned gtk_month;
  _calendar.get_date(year, gtk_month, day);
  month = gtk_month + 1;
}

TemporalValue:: TemporalValue(const Tk::Db::Data::Value &value)
  : _null(Gtk::Stock::APPLY),
    _time(0),
    _time_button(0),
    Value(value.get_type())
{
  const Tk::Db::Data::Datum &d( value.get_datum() );
  
  const odbc::Date *date(0);
  const odbc::Time *time(0);
  
  if (!d.is_null())
    switch(value.get_type())
      {
      case Tk::Db::Data::Date:
	date = &boost::get<Tk::Db::Date>(d.get_value());
	break;
      case Tk::Db::Data::Timestamp:
	{
	  const odbc::Timestamp &timestamp(boost::get<Tk::Db::DateTime>(d.get_value()));
	  date = &timestamp;
	  time = &timestamp;
	  break;
	}
      default:
	bad_type<std::invalid_argument>();
      }
  
  init(value.get_type(), date, time);
}

TemporalValue:: TemporalValue(const Tk::Db::Data::Type &type)
  : _null(Gtk::Stock::APPLY),
    _time(0),
    _time_button(0),
    Value(type)
{
  init(type, 0, 0);
}

void
TemporalValue:: init(const Tk::Db::Data::Type type, const odbc::Date *date, const odbc::Time *time)
{
  if (type == Tk::Db::Data::Timestamp)
    {
      if (time)
	_time = new odbc::Time(*time);
      else
	_time = new odbc::Time;
    }
  
  if (date)
    _date = *date;
  
  init_widgets(type);
}

void
TemporalValue:: init_widgets(const Tk::Db::Data::Type type)
{
  pack_start(_null, false, false);
  _null.set_tooltip_text(_("Whether or not the date datum has a value"));
  _null.signal_toggled().connect( sigc::mem_fun(*this, &TemporalValue::null_toggled) );
  
  pack_start(_date_button, false, true);
  _date_button.signal_clicked().connect( sigc::mem_fun(*this, &TemporalValue::date_clicked) );
  update_date_display();
  
  if (_time)
    {
      _time_button = Gtk::manage( new Gtk::Button() );
      pack_start(*_time_button, false, true);
      _time_button->signal_clicked().connect( sigc::mem_fun(*this, &TemporalValue::time_clicked) );
      update_time_display();
    }
  
  update_sensitivity();
}

TemporalValue:: ~TemporalValue()
{
  if (_time)
    delete _time;
}

void
TemporalValue:: update_date_display()
{
  _date_button.set_label(_date.toString());
}

void
TemporalValue:: update_time_display()
{
  if (_time_button)
    _time_button->set_label(_time->toString());
}

void
TemporalValue:: update_sensitivity()
{
  bool sensitive( !_null.get_active() );
  
  _date_button.set_sensitive(sensitive);
  if (_time_button)
    _time_button->set_sensitive(sensitive);
}

void
TemporalValue:: null_toggled()
{
  update_sensitivity();
  changed();
}

void
TemporalValue:: date_clicked()
{
  DateDialog dialog(_date.getDay(), _date.getMonth(), _date.getYear());

  int result = dialog.run(); 
  if (result == Gtk::RESPONSE_ACCEPT)
    {
      unsigned d, m, y;
      dialog.get_date(d, m, y);
      _date.setDay(d);
      _date.setMonth(m);
      _date.setYear(y);
      
      changed();
      update_date_display();
    }
}

void
TemporalValue:: time_clicked()
{
}

void
TemporalValue:: changed()
{
  if (_null.get_active())
    emit();
  else
    {
      if (!_time)
	emit(_date);
      else
	emit(Tk::Db::DateTime(_date.getYear(),
			      _date.getMonth(),
			      _date.getDay(),
			      _time->getHour(),
			      _time->getMinute(),
			      _time->getSecond()));
    }
}
