/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or 
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * Should you need to contact me, the author, you can do so by
 * e-mail - mail your message to Viktor T. Toth at <vttoth@vttoth.com>.
 *
 * Updated 2007 by Robert Ham for tkresolver
 *
 * Copyright (C) 2007 Robert Ham <rah@bash.sh>
 * Copyright (C) Viktor T. Toth at <vttoth@vttoth.com>
 */

template <typename T>
Matrix<T>::Row:: Row(const Integer::Number &n)
{
  _data = new T[n.get_ui()];
}

template <typename T>
Matrix<T>::Row:: ~Row()
{
  delete[] _data;
}

template <typename T>
T &
Matrix<T>::Row:: operator[](const Integer::Number &c)
{
  return _data[c.get_ui()];
}

template <typename T>
const T &
Matrix<T>::Row:: operator[](const Integer::Number &c) const
{
  return _data[c.get_ui()];
}



template <typename T>
Matrix<T>:: Matrix(const Integer::Number &r, const Integer::Number &c)
  : _nrows(r), _ncols(c)
{
  _rdata = new RowPtr[r.get_ui()];
  for (Integer::Number i(0); i < r; ++i)
    _rdata[i.get_ui()] = new Row(c);
}

template <typename T>
Matrix<T>:: Matrix(const Matrix &op)
  : _nrows(op._nrows), _ncols(op._ncols)
{
  _rdata = new RowPtr[_nrows.get_ui()];
  for (Integer::Number i(0); i < _nrows; ++i)
    _rdata[i.get_ui()] = new Row(_ncols);
  //  Matrix(op._nrows, op._ncols);
  *this = op;
}

template <typename T>
Matrix<T>:: ~Matrix()
{
  delete[] _rdata;
}

template <typename T>
typename Matrix<T>::Row&
Matrix<T>:: operator[](const Integer::Number &r)
{
  return *_rdata[r.get_ui()];
}

template <typename T>
const typename Matrix<T>::Row&
Matrix<T>:: operator[](const Integer::Number &r) const
{
  return *_rdata[r.get_ui()];
}


template <typename T>
Matrix<T> &
Matrix<T>:: operator=(const Matrix<T> &op)
{
  if (_nrows != op._nrows || _ncols != op._ncols)
    throw std::invalid_argument("Matrix size mismatch");
  
  for (int i = 0; i < _nrows; i++)
    for (int j = 0; j < _ncols; j++)
      _rdata[i][j] = op._rdata[i][j];
}

template <typename T>
Matrix<T>
Matrix<T>:: sub(const Integer::Number &r, const Integer::Number &c) const
{
  Matrix<T> t(r, c);
  
  for (Integer::Number i(0); i < (r < _nrows ? r : _nrows); ++i)
    for (Integer::Number j(0); j < (c < _ncols ? c : _ncols); ++j)
      t[i][j] = _rdata[i][j];
  
  return t;
}

template <typename T>
Matrix<T> &
Matrix<T>:: operator*=(const T &op)
{
  for (int i = 0; i < _nrows; i++)
    for (int j = 0; j < _ncols; j++)
      _rdata[i][j] *= op;
  
  return *this;
}

template <typename T>
Matrix<T>
Matrix<T>:: operator*(const T &op) const
{
  Matrix r(*this);
  r *= op;
  return r;
};


template <typename T>
Matrix<T> &
Matrix<T>:: operator/=(const T &op)
{
  for (int i = 0; i < _nrows; i++)
    for (int j = 0; j < _ncols; j++)
      _rdata[i][j] /= op;
  
  return *this;
};


template <typename T>
Matrix<T>
Matrix<T>:: operator/(const T &op) const
{
  Matrix r(*this);
  r /= op;
  return r;
};

template <typename T>
Matrix<T>
Matrix<T>:: operator*(const Matrix<T> &op) const
{
  if (_ncols != op._nrows)
    throw std::invalid_argument("invalid Matrix sizes");
  
  Matrix<T> t(_nrows, op._ncols);
  
  for (int i = 0; i < t._nrows; i++)
    for (int j = 0; j < t._ncols; j++)
      {
	t[i][j] = T(0);
	
	for (int k = 0; k < _ncols; k++)
	  t[i][j] += (*this)[i][k] * op[k][j];
	
      }
  
  return t;
}

template <typename T>
Matrix<T>:: operator Matrix< std::complex<T> > () const
{
  Matrix< std::complex<T> > t(_nrows, _ncols);
  
  for (int i = 0; i < _nrows; i++)
    for (int j = 0; j < _ncols; j++)
      t[i][j] = (*this)[i][j];
  
  return t;
}

template <typename T>
Matrix<T>:: operator Matrix<mpf_class> () const
{
  Matrix<mpf_class> t(_nrows, _ncols);
  mpf_class f;
  
  for (int i = 0; i < _nrows; i++)
    for (int j = 0; j < _ncols; j++)
      t[i][j] = static_cast<mpf_class>( (*this)[i][j] );
  
  return t;
}

template<typename T>
std::ostream &operator<<(std::ostream &os, Matrix<T> const &op)
{
  for (int i = 0; i < op._nrows; i++)
    {
      os << '|';
      
      for (int j = 0; j < op._ncols; j++)
	os << ((j > 0) ? "," : "") << op._rdata[i][j];
      
      os << '|' << std::endl;
    }
  
  return os;
}
