/*
 * TkResolver++ - Psychokinesis testing software
 *
 * Copyright (C) 2004, 2005, 2006, 2007 Robert Ham <rah@bash.sh>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef _GNU_SOURCE
#define _GNU_SOURCE
#endif

#include <wchar.h>

#include <stdlib.h>
#include <getopt.h>

#include "libtkcommon/config.h"
#include "libtkcommon/mbs.h"
#include "libtkcommon/exceptions.h"
#include "libtkcommon/debug.h"
#include "libtkresolver/tkresolver.h"

using namespace Tk;

#define _(x) x

void
print_help ()
{
  std::cout << PACKAGE_NAME << " version " << PACKAGE_VERSION << std::endl;
  std::wcout << L"Copyright (C) 2005 Robert Ham (rah@bash.sh)" << std::endl;
  std::wcout << std::endl;
  std::wcout << L"This program comes with ABSOLUTELY NO WARRANTY.  You are licensed to use it" << std::endl;
  std::wcout << L"under the terms of the GNU General Public License, version 3 or later.  See" << std::endl;
  std::wcout << L"the COPYING file that came with this software for details." << std::endl;
  std::wcout << std::endl;
  std::wcout << L" -h, --help                  " << _("Display this help info") << std::endl;
  std::wcout << L" -w, --width <width>         " << _("Screen width") << std::endl;
  std::wcout << L" -i, --height <height>       " << _("Screen height") << std::endl;
  std::wcout << L" -c, --traces <number>       " << _("Number of traces") << std::endl;
  std::wcout << L" -r, --random <source>       " << _("Select random source (eg, pseudo, hotbits, alea)") << std::endl;
  std::wcout << L" -a, --algorithm <spec>      " << _("Specify probability algorithm") << std::endl;
  std::wcout << L" -b, --bits <number>         " << _("Specify bits per decision") << std::endl;
  std::wcout << L" -t, --time <secs>           " << _("Total time for experiment") << std::endl;
  std::wcout << std::endl;
}

void
print_results (const Result &result)
{
  Line::const_iterator line, next_line;
  for (Result::const_iterator iter = result.begin();
       iter != result.end();
       iter++)
    {
      for (line = iter->begin();
	   line != iter->end();
	   line = next_line)
	{
	  next_line = line;
	  ++next_line;
	  
	  std::cout << *line;
	  
	  if (next_line != iter->end()) 
	  std::cout << ',';
	}
      
      std::cout << std::endl;
    }
}

int
main (int argc, char ** argv)
{
  unsigned width = 1024;
  unsigned height = 768;
  std::wstring retriever_type (L"pseudo");
  std::wstring algo (L"1 4 3");
  unsigned total_time = 1;
  unsigned short ntraces = 1;
  unsigned bit_desc = 1;

  wchar_t * wstr;

  int opt;
  const char * options = "hw:i:r:a:t:b:c:";
  struct option long_options[] = {
    { "help",        no_argument,       NULL, 'h' },
    { "width",       required_argument, NULL, 'w' },
    { "height",      required_argument, NULL, 'i' },
    { "random",      required_argument, NULL, 'r' },
    { "algorithm",   required_argument, NULL, 'a' },
    { "bits",        required_argument, NULL, 'b' },
    { "time",        required_argument, NULL, 't' },
    { "traces",      required_argument, NULL, 'c' },
    { 0, 0, 0, 0 }
  };
  
  while ((opt = getopt_long (argc, argv, options, long_options, NULL)) != -1) {
    switch (opt) {
      
    case 'w':
      width = strtoul(optarg, NULL, 10);
      break;

    case 'i':
      height = strtoul(optarg, NULL, 10);
      break;

    case 'c':
      ntraces = strtoul(optarg, NULL, 10);
      break;

    case 'b':
      bit_desc = strtoul(optarg, NULL, 10);
      break;

    case 'r':
      wstr = mbstowcs_alloc(optarg);
      retriever_type = std::wstring(wstr);
      free(wstr);
      break;

    case 'a':
      wstr = mbstowcs_alloc(optarg);
      algo = std::wstring(wstr);
      free(wstr);
      break;

    case 't':
      total_time = strtoul(optarg, NULL, 10);
      break;

    case 'h':
      print_help ();
      exit (0);
      break;
    case ':':
    case '?':
      print_help ();
      exit (1);
      break;
    }
  }


  try
    {
      TKResolver::TKResolver tkresolver(retriever_type, algo, bit_desc, ntraces, total_time, width, height);

      DEBUG_OUT << "running tkresolver...";
      tkresolver.run();
      DEBUG_OUT << "done"
		<< std::endl;

      print_results(tkresolver.get_result());
    }
  catch (UnixError e)
    {
      std::wcerr << L"Uncaught Unix exception ("
		 << String(e.what())
		 << L") - aborting"
		 << std::endl;
      exit(1);
    }
  catch (std::invalid_argument ia)
    {
      std::wcerr << L"Exception: invalid argument (";
      std::cerr  << ia.what();
      std::wcerr << L") - aborting"
		 << std::endl;
      exit(1);
    }
  catch (std::underflow_error ue)
    {
      std::wcerr << L"Exception: underflow error (";
      std::cerr  << ue.what();
      std::wcerr << L") - aborting"
		 << std::endl;
      exit(1);
    }
  catch (std::runtime_error re)
    {
      std::wcerr << L"Exception: runtime error (";
      std::cerr  << re.what();
      std::wcerr << L") - aborting"
		 << std::endl;
      exit(1);
    }
  catch (std::exception except)
    {
      std::wcerr << L"Uncaught exception (";
      std::cerr  << except.what();
      std::wcerr << L") - aborting"
		 << std::endl;
      exit(1);
    }
  catch (...)
    {
      std::wcerr << L"Uncaught unknown exception - aborting"
		 << std::endl;
      exit(1);
    }
  
  exit (0);
}
